@@ -15,8 +15,8 @@
  *  GNU Library General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *  along with this program; if not, write to the Free Software Foundation,
+ *  Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
  *
  *  Random portions taken from or inspired by the original xfdesktop for xfce4:
  *     Copyright (C) 2002-2003 Jasper Huijsmans (huysmans@users.sourceforge.net)
@@ -28,21 +28,21 @@
 
 #include <glib.h>
 #include <gdk/gdk.h>
+#include <gtk/gtk.h>
 
 #include <X11/Xlib.h>
 
 #define XFDESKTOP_CHANNEL        "xfce4-desktop"
 #define DEFAULT_BACKDROP         DATADIR "/backgrounds/xfce/xfce-blue.jpg"
-#define DEFAULT_BACKDROP_LIST    "xfce4/desktop/backdrop.list"
 #define DEFAULT_ICON_FONT_SIZE   12
-#define DEFAULT_ICON_SIZE        32
+#define DEFAULT_ICON_SIZE        48
 #define ITHEME_FLAGS             (GTK_ICON_LOOKUP_USE_BUILTIN \
-                                  | GTK_ICON_LOOKUP_GENERIC_FALLBACK \
-                                  | GTK_ICON_LOOKUP_FORCE_SIZE)
+                                  | GTK_ICON_LOOKUP_GENERIC_FALLBACK)
 
 #define LIST_TEXT                "# xfce backdrop list"
 #define XFDESKTOP_SELECTION_FMT  "XFDESKTOP_SELECTION_%d"
 #define XFDESKTOP_IMAGE_FILE_FMT "XFDESKTOP_IMAGE_FILE_%d"
+#define XFDESKTOP_RC_VERSION_STAMP "xfdesktop-version-4.10.3+-rcfile_format"
 
 #define RELOAD_MESSAGE     "reload"
 #define MENU_MESSAGE       "menu"
@@ -50,6 +50,20 @@
 #define ARRANGE_MESSAGE    "arrange"
 #define QUIT_MESSAGE       "quit"
 
+#define SINGLE_WORKSPACE_MODE     "/backdrop/single-workspace-mode"
+#define SINGLE_WORKSPACE_NUMBER   "/backdrop/single-workspace-number"
+
+#define DESKTOP_ICONS_SHOW_THUMBNAILS        "/desktop-icons/show-thumbnails"
+#define DESKTOP_ICONS_SHOW_NETWORK_REMOVABLE "/desktop-icons/file-icons/show-network-removable"
+#define DESKTOP_ICONS_SHOW_DEVICE_REMOVABLE  "/desktop-icons/file-icons/show-device-removable"
+#define DESKTOP_ICONS_SHOW_UNKNWON_REMOVABLE "/desktop-icons/file-icons/show-unknown-removable"
+#define DESKTOP_ICONS_SHOW_HOME              "/desktop-icons/file-icons/show-home"
+#define DESKTOP_ICONS_SHOW_TRASH             "/desktop-icons/file-icons/show-trash"
+#define DESKTOP_ICONS_SHOW_FILESYSTEM        "/desktop-icons/file-icons/show-filesystem"
+#define DESKTOP_ICONS_SHOW_REMOVABLE         "/desktop-icons/file-icons/show-removable"
+
+#define DESKTOP_MENU_MAX_TEMPLATE_FILES     "/desktop-menu/max-template-files"
+
 /**
  * File information namespaces queried for #GFileInfo objects.
  */
@@ -62,7 +76,8 @@
   "time::*," \
   "thumbnail::*," \
   "trash::*," \
-  "unix::*"
+  "unix::*," \
+  "metadata::*"
 
 /**
  * Filesystem information namespaces queried for #GFileInfo * objects.
@@ -72,22 +87,15 @@
 
 G_BEGIN_DECLS
 
-gchar **xfdesktop_backdrop_list_load(const gchar *filename,
-                                     gint *n_items,
-                                     GError **error);
-gboolean xfdesktop_backdrop_list_save(const gchar *filename,
-                                      gchar * const *files,
-                                      GError **error);
-gchar *xfdesktop_backdrop_list_choose_random(const gchar *filename,
-                                             GError **error);
-gboolean xfdesktop_backdrop_list_is_valid(const gchar *filename);
+gint xfdesktop_compare_paths(GFile *a, GFile *b);
 
 gboolean xfdesktop_image_file_is_valid(const gchar *filename);
 
-gboolean xfdesktop_check_is_running(Window *xid);
-void xfdesktop_send_client_message(Window xid, const gchar *msg);
+gchar *xfdesktop_get_file_mimetype(const gchar *file);
 
-gboolean xfdesktop_popup_grab_available(GdkWindow *win, guint32 timestamp);
+gint xfce_translate_image_styles(gint input);
+
+guint32 xfdesktop_popup_keyboard_grab_available(GdkWindow *win);
 
 G_END_DECLS
 
