@@ -14,8 +14,8 @@
  *  GNU Library General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *  along with this program; if not, write to the Free Software Foundation,
+ *  Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
  *
  *  Random portions taken from or inspired by the original xfdesktop for xfce4:
  *     Copyright (C) 2002-2003 Jasper Huijsmans (huysmans@users.sourceforge.net)
@@ -25,18 +25,16 @@
 #ifndef _XFDESKTOP__MENU_H_
 #define _XFDESKTOP__MENU_H_
 
-#include <gdk/gdkscreen.h>
-#include <gtk/gtkwidget.h>
+#include "xfce-desktop.h"
 
-#include <libxfce4mcs/mcs-client.h>
+#include <xfconf/xfconf.h>
 
 G_BEGIN_DECLS
 
-void menu_init(McsClient *mcs_client);
-gboolean menu_settings_changed(McsClient *client, McsAction action, McsSetting *setting, gpointer user_data);
-void popup_desktop_menu(GdkScreen *gscreen, gint button, guint32 time);
-void menu_reload();
-void menu_cleanup();
+void menu_init(XfconfChannel *channel);
+void menu_attach(XfceDesktop *desktop);
+void menu_reload(void);
+void menu_cleanup(void);
 
 G_END_DECLS
 
