@@ -14,8 +14,8 @@
  *  GNU Library General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *  along with this program; if not, write to the Free Software Foundation,
+ *  Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
  */
 
 #ifndef __XFDESKTOP_FILE_ICON_MANAGER_H__
@@ -55,22 +55,10 @@ GType xfdesktop_file_icon_manager_get_type(void) G_GNUC_CONST;
 XfdesktopIconViewManager *xfdesktop_file_icon_manager_new(GFile *folder,
                                                           XfconfChannel *channel);
 
-void xfdesktop_file_icon_manager_set_show_removable_media(XfdesktopFileIconManager *manager,
-                                                          gboolean show_removable_media);
-gboolean xfdesktop_file_icon_manager_get_show_removable_media(XfdesktopFileIconManager *manager);
-
-void xfdesktop_file_icon_manager_set_show_special_file(XfdesktopFileIconManager *manager,
-                                                       XfdesktopSpecialFileIconType type,
-                                                       gboolean show_special_file);
-gboolean xfdesktop_file_icon_manager_get_show_special_file(XfdesktopFileIconManager *manager,
-                                                           XfdesktopSpecialFileIconType type);
-void xfdesktop_file_icon_manager_set_show_thumbnails(XfdesktopFileIconManager *manager,
-                                                     gboolean show_thumbnails);
-gboolean xfdesktop_file_icon_manager_get_show_thumbnails(XfdesktopFileIconManager *manager);
-
 gboolean xfdesktop_file_icon_manager_get_cached_icon_position(
                                                     XfdesktopFileIconManager *fmanager,
                                                     const gchar *name,
+                                                    const gchar *identifier,
                                                     gint16 *row,
                                                     gint16 *col);
 
