@@ -14,8 +14,8 @@
  *  GNU Library General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *  along with this program; if not, write to the Free Software Foundation,
+ *  Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
  */
 
 #ifdef HAVE_CONFIG_H
@@ -37,10 +37,13 @@ struct _XfdesktopIconPrivate
     gint16 row;
     gint16 col;
 
-    gboolean extents_dirty;
     GdkRectangle pixbuf_extents;
     GdkRectangle text_extents;
     GdkRectangle total_extents;
+
+    GdkPixbuf *pix, *tooltip_pix;
+    gint cur_pix_width, cur_pix_height;
+    gint cur_tooltip_pix_width, cur_tooltip_pix_height;
 };
 
 enum {
@@ -55,6 +58,7 @@ enum {
 
 static guint __signals[SIG_N_SIGNALS] = { 0, };
 
+static void xfdesktop_icon_finalize(GObject *obj);
 
 G_DEFINE_ABSTRACT_TYPE(XfdesktopIcon, xfdesktop_icon, G_TYPE_OBJECT)
 
@@ -62,8 +66,12 @@ G_DEFINE_ABSTRACT_TYPE(XfdesktopIcon, xfdesktop_icon, G_TYPE_OBJECT)
 static void
 xfdesktop_icon_class_init(XfdesktopIconClass *klass)
 {
+    GObjectClass *gobject_class = (GObjectClass *)klass;
+
     g_type_class_add_private(klass, sizeof(XfdesktopIconPrivate));
-    
+
+    gobject_class->finalize = xfdesktop_icon_finalize;
+
     __signals[SIG_PIXBUF_CHANGED] = g_signal_new("pixbuf-changed",
                                                  XFDESKTOP_TYPE_ICON,
                                                  G_SIGNAL_RUN_LAST,
@@ -116,9 +124,15 @@ xfdesktop_icon_init(XfdesktopIcon *icon)
 {
     icon->priv = G_TYPE_INSTANCE_GET_PRIVATE(icon, XFDESKTOP_TYPE_ICON,
                                              XfdesktopIconPrivate);
-    icon->priv->extents_dirty = TRUE;
 }
 
+static void
+xfdesktop_icon_finalize(GObject *obj)
+{
+    XfdesktopIcon *icon = XFDESKTOP_ICON(obj);
+
+    xfdesktop_icon_invalidate_pixbuf(icon);
+}
 
 void
 xfdesktop_icon_set_position(XfdesktopIcon *icon,
@@ -158,7 +172,6 @@ xfdesktop_icon_set_extents(XfdesktopIcon *icon,
     icon->priv->pixbuf_extents = *pixbuf_extents;
     icon->priv->text_extents = *text_extents;
     icon->priv->total_extents = *total_extents;
-    icon->priv->extents_dirty = FALSE;
 }
 
 gboolean
@@ -169,9 +182,6 @@ xfdesktop_icon_get_extents(XfdesktopIcon *icon,
 {
     g_return_val_if_fail(XFDESKTOP_IS_ICON(icon), FALSE);
 
-    if(icon->priv->extents_dirty)
-        return FALSE;
-
     if(pixbuf_extents)
         *pixbuf_extents = icon->priv->pixbuf_extents;
     if(text_extents)
@@ -182,32 +192,33 @@ xfdesktop_icon_get_extents(XfdesktopIcon *icon,
     return TRUE;
 }
 
-void
-xfdesktop_icon_mark_extents_dirty(XfdesktopIcon *icon)
-{
-    g_return_if_fail(XFDESKTOP_IS_ICON(icon));
-
-    icon->priv->extents_dirty = TRUE;
-}
-
-
-
 /*< required >*/
 GdkPixbuf *
 xfdesktop_icon_peek_pixbuf(XfdesktopIcon *icon,
-                           gint size)
+                           gint width, gint height)
 {
     XfdesktopIconClass *klass;
     
     g_return_val_if_fail(XFDESKTOP_IS_ICON(icon), NULL);
     klass = XFDESKTOP_ICON_GET_CLASS(icon);
     g_return_val_if_fail(klass->peek_pixbuf, NULL);
-    
-    return klass->peek_pixbuf(icon, size);
+
+    if(width != icon->priv->cur_pix_width || height != icon->priv->cur_pix_height)
+        xfdesktop_icon_invalidate_regular_pixbuf(icon);
+
+    if(icon->priv->pix == NULL) {
+        icon->priv->cur_pix_width = width;
+        icon->priv->cur_pix_height = height;
+
+        /* Generate a new pixbuf */
+        icon->priv->pix = klass->peek_pixbuf(icon, width, height);
+    }
+
+    return icon->priv->pix;
 }
 
 /*< required >*/
-G_CONST_RETURN gchar *
+const gchar *
 xfdesktop_icon_peek_label(XfdesktopIcon *icon)
 {
     XfdesktopIconClass *klass;
@@ -219,6 +230,22 @@ xfdesktop_icon_peek_label(XfdesktopIcon *icon)
     return klass->peek_label(icon);
 }
 
+/*< required >*/
+gchar *
+xfdesktop_icon_get_identifier(XfdesktopIcon *icon)
+{
+    XfdesktopIconClass *klass;
+
+    g_return_val_if_fail(XFDESKTOP_IS_ICON(icon), NULL);
+
+    klass = XFDESKTOP_ICON_GET_CLASS(icon);
+
+    if(!klass->get_identifier)
+        return NULL;
+
+    return klass->get_identifier(icon);
+}
+
 /*< optional; drags aren't allowed if not provided >*/
 GdkDragAction
 xfdesktop_icon_get_allowed_drag_actions(XfdesktopIcon *icon)
@@ -237,7 +264,8 @@ xfdesktop_icon_get_allowed_drag_actions(XfdesktopIcon *icon)
 
 /*< optional; drops aren't allowed if not provided >*/
 GdkDragAction
-xfdesktop_icon_get_allowed_drop_actions(XfdesktopIcon *icon)
+xfdesktop_icon_get_allowed_drop_actions(XfdesktopIcon *icon,
+                                        GdkDragAction *suggested_action)
 {
     XfdesktopIconClass *klass;
     
@@ -245,10 +273,13 @@ xfdesktop_icon_get_allowed_drop_actions(XfdesktopIcon *icon)
     
     klass = XFDESKTOP_ICON_GET_CLASS(icon);
     
-    if(!klass->get_allowed_drop_actions)
+    if(!klass->get_allowed_drop_actions) {
+        if(suggested_action)
+            *suggested_action = 0;
         return 0;
-    
-    return klass->get_allowed_drop_actions(icon);
+    }
+
+    return klass->get_allowed_drop_actions(icon, suggested_action);
 }
 
 /*< optional; required if get_allowed_drop_actions() can return nonzero >*/
@@ -267,7 +298,32 @@ xfdesktop_icon_do_drop_dest(XfdesktopIcon *icon,
 }
 
 /*< optional >*/
-G_CONST_RETURN gchar *
+GdkPixbuf *
+xfdesktop_icon_peek_tooltip_pixbuf(XfdesktopIcon *icon,
+                                   gint width, gint height)
+{
+    XfdesktopIconClass *klass;
+
+    g_return_val_if_fail(XFDESKTOP_IS_ICON(icon), NULL);
+    klass = XFDESKTOP_ICON_GET_CLASS(icon);
+    g_return_val_if_fail(klass->peek_tooltip_pixbuf, NULL);
+
+    if(width != icon->priv->cur_tooltip_pix_width || height != icon->priv->cur_tooltip_pix_height)
+        xfdesktop_icon_invalidate_tooltip_pixbuf(icon);
+
+    if(icon->priv->tooltip_pix == NULL) {
+        icon->priv->cur_tooltip_pix_width = width;
+        icon->priv->cur_tooltip_pix_height = height;
+
+        /* Generate a new pixbuf */
+        icon->priv->tooltip_pix = klass->peek_tooltip_pixbuf(icon, width, height);
+    }
+
+    return icon->priv->tooltip_pix;
+}
+
+/*< optional >*/
+const gchar *
 xfdesktop_icon_peek_tooltip(XfdesktopIcon *icon)
 {
     XfdesktopIconClass *klass;
@@ -282,6 +338,38 @@ xfdesktop_icon_peek_tooltip(XfdesktopIcon *icon)
     return klass->peek_tooltip(icon);
 }
 
+
+/*< optional >*/
+void xfdesktop_icon_delete_thumbnail(XfdesktopIcon *icon)
+{
+    XfdesktopIconClass *klass;
+
+    g_return_if_fail(XFDESKTOP_IS_ICON(icon));
+
+    klass = XFDESKTOP_ICON_GET_CLASS(icon);
+
+    if(!klass->delete_thumbnail_file)
+        return;
+
+    klass->delete_thumbnail_file(icon);
+}
+
+/*< optional >*/
+void
+xfdesktop_icon_set_thumbnail_file(XfdesktopIcon *icon, GFile *file)
+{
+    XfdesktopIconClass *klass;
+
+    g_return_if_fail(XFDESKTOP_IS_ICON(icon));
+
+    klass = XFDESKTOP_ICON_GET_CLASS(icon);
+
+    if(!klass->set_thumbnail_file)
+        return;
+
+    klass->set_thumbnail_file(icon, file);
+}
+
 /*< optional >*/
 gboolean
 xfdesktop_icon_populate_context_menu(XfdesktopIcon *icon,
@@ -306,13 +394,37 @@ xfdesktop_icon_peek_icon_view(XfdesktopIcon *icon)
     return g_object_get_data(G_OBJECT(icon), "--xfdesktop-icon-view");
 }
 
+void
+xfdesktop_icon_invalidate_regular_pixbuf(XfdesktopIcon *icon)
+{
+    if(icon->priv->pix) {
+        g_object_unref(G_OBJECT(icon->priv->pix));
+        icon->priv->pix = NULL;
+    }
+}
+
+void
+xfdesktop_icon_invalidate_tooltip_pixbuf(XfdesktopIcon *icon)
+{
+    if(icon->priv->tooltip_pix) {
+        g_object_unref(G_OBJECT(icon->priv->tooltip_pix));
+        icon->priv->tooltip_pix = NULL;
+    }
+}
+
+void
+xfdesktop_icon_invalidate_pixbuf(XfdesktopIcon *icon)
+{
+    xfdesktop_icon_invalidate_regular_pixbuf(icon);
+    xfdesktop_icon_invalidate_tooltip_pixbuf(icon);
+}
+
 /*< signal triggers >*/
 
 void
 xfdesktop_icon_pixbuf_changed(XfdesktopIcon *icon)
 {
     g_return_if_fail(XFDESKTOP_IS_ICON(icon));
-    xfdesktop_icon_mark_extents_dirty(icon);
     g_signal_emit(icon, __signals[SIG_PIXBUF_CHANGED], 0);
 }
 
@@ -320,7 +432,6 @@ void
 xfdesktop_icon_label_changed(XfdesktopIcon *icon)
 {
     g_return_if_fail(XFDESKTOP_IS_ICON(icon));
-    xfdesktop_icon_mark_extents_dirty(icon);
     g_signal_emit(icon, __signals[SIG_LABEL_CHANGED], 0);
 }
 
@@ -328,7 +439,6 @@ void
 xfdesktop_icon_position_changed(XfdesktopIcon *icon)
 {
     g_return_if_fail(XFDESKTOP_IS_ICON(icon));
-    xfdesktop_icon_mark_extents_dirty(icon);
     g_signal_emit(icon, __signals[SIG_POS_CHANGED], 0);
 }
 
@@ -337,7 +447,6 @@ void
 xfdesktop_icon_selected(XfdesktopIcon *icon)
 {
     g_return_if_fail(XFDESKTOP_IS_ICON(icon));
-    xfdesktop_icon_mark_extents_dirty(icon);
     g_signal_emit(G_OBJECT(icon), __signals[SIG_SELECTED], 0, NULL);
 }
 
