@@ -16,8 +16,8 @@
  *  GNU Library General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *  along with this program; if not, write to the Free Software Foundation,
+ *  Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
  */
 
 #ifdef HAVE_CONFIG_H
@@ -55,19 +55,21 @@
 #include "xfdesktop-regular-file-icon.h"
 
 #define EMBLEM_SYMLINK  "emblem-symbolic-link"
+#define EMBLEM_READONLY "emblem-readonly"
 
 struct _XfdesktopRegularFileIconPrivate
 {
-    GdkPixbuf *pix;
     gchar *display_name;
     gchar *tooltip;
     guint pix_opacity;
-    gint cur_pix_size;
     GFileInfo *file_info;
     GFileInfo *filesystem_info;
     GFile *file;
     GFile *thumbnail_file;
+    GFileMonitor *monitor;
     GdkScreen *gscreen;
+    XfdesktopFileIconManager *fmanager;
+    gboolean show_thumbnails;
 };
 
 static void xfdesktop_regular_file_icon_finalize(GObject *obj);
@@ -76,11 +78,15 @@ static void xfdesktop_regular_file_icon_set_thumbnail_file(XfdesktopIcon *icon,
 static void xfdesktop_regular_file_icon_delete_thumbnail_file(XfdesktopIcon *icon);
 
 static GdkPixbuf *xfdesktop_regular_file_icon_peek_pixbuf(XfdesktopIcon *icon,
-                                                          gint size);
-static G_CONST_RETURN gchar *xfdesktop_regular_file_icon_peek_label(XfdesktopIcon *icon);
-static G_CONST_RETURN gchar *xfdesktop_regular_file_icon_peek_tooltip(XfdesktopIcon *icon);
+                                                          gint width, gint height);
+static const gchar *xfdesktop_regular_file_icon_peek_label(XfdesktopIcon *icon);
+static gchar *xfdesktop_regular_file_icon_get_identifier(XfdesktopIcon *icon);
+static GdkPixbuf *xfdesktop_regular_file_icon_peek_tooltip_pixbuf(XfdesktopIcon *icon,
+                                                                  gint width, gint height);
+static const gchar *xfdesktop_regular_file_icon_peek_tooltip(XfdesktopIcon *icon);
 static GdkDragAction xfdesktop_regular_file_icon_get_allowed_drag_actions(XfdesktopIcon *icon);
-static GdkDragAction xfdesktop_regular_file_icon_get_allowed_drop_actions(XfdesktopIcon *icon);
+static GdkDragAction xfdesktop_regular_file_icon_get_allowed_drop_actions(XfdesktopIcon *icon,
+                                                                          GdkDragAction *suggested_action);
 static gboolean xfdesktop_regular_file_icon_do_drop_dest(XfdesktopIcon *icon,
                                                          XfdesktopIcon *src_icon,
                                                          GdkDragAction action);
@@ -94,12 +100,7 @@ static gboolean xfdesktop_regular_file_can_write_parent(XfdesktopFileIcon *icon)
 
 #ifdef HAVE_THUNARX
 static void xfdesktop_regular_file_icon_tfi_init(ThunarxFileInfoIface *iface);
-#endif
 
-static inline void xfdesktop_regular_file_icon_invalidate_pixbuf(XfdesktopRegularFileIcon *icon);
-
-
-#ifdef HAVE_THUNARX
 G_DEFINE_TYPE_EXTENDED(XfdesktopRegularFileIcon, xfdesktop_regular_file_icon,
                        XFDESKTOP_TYPE_FILE_ICON, 0,
                        G_IMPLEMENT_INTERFACE(THUNARX_TYPE_FILE_INFO,
@@ -125,6 +126,8 @@ xfdesktop_regular_file_icon_class_init(XfdesktopRegularFileIconClass *klass)
     
     icon_class->peek_pixbuf = xfdesktop_regular_file_icon_peek_pixbuf;
     icon_class->peek_label = xfdesktop_regular_file_icon_peek_label;
+    icon_class->get_identifier = xfdesktop_regular_file_icon_get_identifier;
+    icon_class->peek_tooltip_pixbuf = xfdesktop_regular_file_icon_peek_tooltip_pixbuf;
     icon_class->peek_tooltip = xfdesktop_regular_file_icon_peek_tooltip;
     icon_class->get_allowed_drag_actions = xfdesktop_regular_file_icon_get_allowed_drag_actions;
     icon_class->get_allowed_drop_actions = xfdesktop_regular_file_icon_get_allowed_drop_actions;
@@ -157,11 +160,8 @@ xfdesktop_regular_file_icon_finalize(GObject *obj)
     GtkIconTheme *itheme = gtk_icon_theme_get_for_screen(icon->priv->gscreen);
     
     g_signal_handlers_disconnect_by_func(G_OBJECT(itheme),
-                                         G_CALLBACK(xfdesktop_regular_file_icon_invalidate_pixbuf),
+                                         G_CALLBACK(xfdesktop_icon_invalidate_pixbuf),
                                          icon);
-
-    if(icon->priv->pix)
-        g_object_unref(G_OBJECT(icon->priv->pix));
     
     if(icon->priv->file_info)
         g_object_unref(icon->priv->file_info);
@@ -172,13 +172,16 @@ xfdesktop_regular_file_icon_finalize(GObject *obj)
     g_object_unref(icon->priv->file);
     
     g_free(icon->priv->display_name);
-    
+
     if(icon->priv->tooltip)
         g_free(icon->priv->tooltip);
 
     if(icon->priv->thumbnail_file)
         g_object_unref(icon->priv->thumbnail_file);
-    
+
+    if(icon->priv->monitor)
+        g_object_unref(icon->priv->monitor);
+
     G_OBJECT_CLASS(xfdesktop_regular_file_icon_parent_class)->finalize(obj);
 }
 
@@ -199,15 +202,6 @@ xfdesktop_regular_file_icon_tfi_init(ThunarxFileInfoIface *iface)
 }
 #endif
 
-static inline void
-xfdesktop_regular_file_icon_invalidate_pixbuf(XfdesktopRegularFileIcon *icon)
-{
-    if(icon->priv->pix) {
-        g_object_unref(G_OBJECT(icon->priv->pix));
-        icon->priv->pix = NULL;
-    }
-}
-
 static void
 xfdesktop_regular_file_icon_delete_thumbnail_file(XfdesktopIcon *icon)
 {
@@ -223,8 +217,10 @@ xfdesktop_regular_file_icon_delete_thumbnail_file(XfdesktopIcon *icon)
         file_icon->priv->thumbnail_file = NULL;
     }
 
-    xfdesktop_regular_file_icon_invalidate_pixbuf(file_icon);
-    xfdesktop_icon_pixbuf_changed(XFDESKTOP_ICON(icon));
+    xfdesktop_file_icon_invalidate_icon(XFDESKTOP_FILE_ICON(icon));
+
+    xfdesktop_icon_invalidate_pixbuf(icon);
+    xfdesktop_icon_pixbuf_changed(icon);
 }
 
 static void
@@ -242,127 +238,294 @@ xfdesktop_regular_file_icon_set_thumbnail_file(XfdesktopIcon *icon, GFile *file)
 
     file_icon->priv->thumbnail_file = file;
 
-    xfdesktop_regular_file_icon_invalidate_pixbuf(file_icon);
-    xfdesktop_icon_pixbuf_changed(XFDESKTOP_ICON(icon));
+    xfdesktop_file_icon_invalidate_icon(XFDESKTOP_FILE_ICON(icon));
+
+    xfdesktop_icon_invalidate_pixbuf(icon);
+    xfdesktop_icon_pixbuf_changed(icon);
+}
+
+
+static void
+cb_show_thumbnails_notify(GObject *gobject,
+                          GParamSpec *pspec,
+                          gpointer user_data)
+{
+    XfdesktopRegularFileIcon *regular_file_icon;
+    gboolean show_thumbnails = FALSE;
+
+    TRACE("entering");
+
+    if(!user_data || !XFDESKTOP_IS_REGULAR_FILE_ICON(user_data))
+        return;
+
+    regular_file_icon = XFDESKTOP_REGULAR_FILE_ICON(user_data);
+
+    g_object_get(regular_file_icon->priv->fmanager, "show-thumbnails", &show_thumbnails, NULL);
+
+    if(regular_file_icon->priv->show_thumbnails != show_thumbnails) {
+        DBG("show-thumbnails changed! now: %s", show_thumbnails ? "TRUE" : "FALSE");
+        regular_file_icon->priv->show_thumbnails = show_thumbnails;
+        xfdesktop_file_icon_invalidate_icon(XFDESKTOP_FILE_ICON(regular_file_icon));
+        xfdesktop_icon_invalidate_pixbuf(XFDESKTOP_ICON(regular_file_icon));
+        xfdesktop_icon_pixbuf_changed(XFDESKTOP_ICON(regular_file_icon));
+    }
+}
+
+
+/* builds a folder/file path and then tests if that file is a valid image.
+ * returns the file location if it does, NULL if it doesn't */
+static gchar *
+xfdesktop_check_file_is_valid(const gchar *folder, const gchar *file)
+{
+    gchar *path = g_strconcat(folder, "/", file, NULL);
+
+    if(gdk_pixbuf_get_file_info(path, NULL, NULL) == NULL) {
+        g_free(path);
+        path = NULL;
+    }
+
+    return path;
+}
+
+static gchar *
+xfdesktop_load_icon_location_from_folder(XfdesktopFileIcon *icon)
+{
+    gchar *icon_file = g_file_get_path(xfdesktop_file_icon_peek_file(icon));
+    gchar *path;
+
+    g_return_val_if_fail(icon_file, NULL);
+
+    /* So much for standards */
+    path = xfdesktop_check_file_is_valid(icon_file, "Folder.jpg");
+    if(path == NULL) {
+        path = xfdesktop_check_file_is_valid(icon_file, "folder.jpg");
+    }
+    if(path == NULL) {
+        path = xfdesktop_check_file_is_valid(icon_file, "Folder.JPG");
+    }
+    if(path == NULL) {
+        path = xfdesktop_check_file_is_valid(icon_file, "folder.JPG");
+    }
+    if(path == NULL) {
+        path = xfdesktop_check_file_is_valid(icon_file, "folder.jpeg");
+    }
+    if(path == NULL) {
+        path = xfdesktop_check_file_is_valid(icon_file, "folder.JPEG");
+    }
+    if(path == NULL) {
+        path = xfdesktop_check_file_is_valid(icon_file, "Folder.JPEG");
+    }
+    if(path == NULL) {
+        path = xfdesktop_check_file_is_valid(icon_file, "Folder.jpeg");
+    }
+    if(path == NULL) {
+        path = xfdesktop_check_file_is_valid(icon_file, "Cover.jpg");
+    }
+    if(path == NULL) {
+        path = xfdesktop_check_file_is_valid(icon_file, "cover.jpg");
+    }
+    if(path == NULL) {
+        path = xfdesktop_check_file_is_valid(icon_file, "Cover.jpeg");
+    }
+    if(path == NULL) {
+        path = xfdesktop_check_file_is_valid(icon_file, "cover.jpeg");
+    }
+    if(path == NULL) {
+        path = xfdesktop_check_file_is_valid(icon_file, "albumart.jpg");
+    }
+    if(path == NULL) {
+        path = xfdesktop_check_file_is_valid(icon_file, "albumart.jpeg");
+    }
+    if(path == NULL) {
+        path = xfdesktop_check_file_is_valid(icon_file, "fanart.jpg");
+    }
+    if(path == NULL) {
+        path = xfdesktop_check_file_is_valid(icon_file, "Fanart.jpg");
+    }
+    if(path == NULL) {
+        path = xfdesktop_check_file_is_valid(icon_file, "fanart.JPG");
+    }
+    if(path == NULL) {
+        path = xfdesktop_check_file_is_valid(icon_file, "Fanart.JPG");
+    }
+    if(path == NULL) {
+        path = xfdesktop_check_file_is_valid(icon_file, "FANART.JPG");
+    }
+    if(path == NULL) {
+        path = xfdesktop_check_file_is_valid(icon_file, "FANART.jpg");
+    }
+
+    g_free(icon_file);
+
+    /* the file *should* already be a thumbnail */
+    return path;
+}
+
+static GIcon *
+xfdesktop_load_icon_from_desktop_file(XfdesktopRegularFileIcon *regular_icon)
+{
+    gchar *contents;
+    gsize length;
+    GIcon *gicon = NULL;
+
+    /* try to load the file into memory */
+    if(g_file_load_contents(regular_icon->priv->file, NULL, &contents, &length,
+                            NULL, NULL))
+    {
+        /* allocate a new key file */
+        GKeyFile *key_file = g_key_file_new();
+
+        /* try to parse the key file from the contents of the file */
+        if (g_key_file_load_from_data(key_file, contents, length, 0, NULL)) {
+            gchar *icon_name;
+            /* try to determine the custom icon name */
+            icon_name = g_key_file_get_string(key_file,
+                                              G_KEY_FILE_DESKTOP_GROUP,
+                                              G_KEY_FILE_DESKTOP_KEY_ICON,
+                                              NULL);
+
+            if(icon_name && g_file_test(icon_name, G_FILE_TEST_IS_REGULAR)) {
+                /* icon_name is an absolute path, create it as a file icon */
+                gicon = g_file_icon_new(g_file_new_for_path(icon_name));
+            } else if(icon_name) {
+                /* otherwise create a themed icon for it */
+                gicon = g_themed_icon_new(icon_name);
+            }
+
+            if(icon_name)
+                g_free(icon_name);
+        }
+
+        /* free key file and in-memory data */
+        g_key_file_free(key_file);
+        g_free(contents);
+    }
+
+    return gicon;
+}
+
+static GIcon *
+xfdesktop_regular_file_icon_load_icon(XfdesktopIcon *icon)
+{
+    XfdesktopRegularFileIcon *regular_icon = XFDESKTOP_REGULAR_FILE_ICON(icon);
+    XfdesktopFileIcon *file_icon = XFDESKTOP_FILE_ICON(icon);
+    GIcon *gicon = NULL;
+
+    TRACE("entering");
+
+    /* Try to load the icon referenced in the .desktop file */
+    if(xfdesktop_file_utils_is_desktop_file(regular_icon->priv->file_info)) {
+        gicon = xfdesktop_load_icon_from_desktop_file(regular_icon);
+
+    } else if(g_file_info_get_file_type(regular_icon->priv->file_info) == G_FILE_TYPE_DIRECTORY) {
+        /* Try to load a thumbnail from the standard folder image locations */
+        gchar *thumbnail_file = NULL;
+
+        if(regular_icon->priv->show_thumbnails)
+            thumbnail_file = xfdesktop_load_icon_location_from_folder(file_icon);
+
+        if(thumbnail_file) {
+            /* If there's a folder thumbnail, use it */
+            regular_icon->priv->thumbnail_file = g_file_new_for_path(thumbnail_file);
+            gicon = g_file_icon_new(regular_icon->priv->thumbnail_file);
+            g_free(thumbnail_file);
+        }
+
+    } else {
+        /* If we have a thumbnail then they are enabled, use it. */
+        if(regular_icon->priv->thumbnail_file) {
+            gchar *file = g_file_get_path(regular_icon->priv->file);
+            gchar *mimetype = xfdesktop_get_file_mimetype(file);
+
+            /* Don't use thumbnails for svg, use the file itself */
+            if(g_strcmp0(mimetype, "image/svg+xml") == 0)
+                gicon = g_file_icon_new(regular_icon->priv->file);
+            else
+                gicon = g_file_icon_new(regular_icon->priv->thumbnail_file);
+
+            g_free(mimetype);
+            g_free(file);
+        }
+    }
+
+    /* If we still don't have an icon, use the default */
+    if(!G_IS_ICON(gicon)) {
+        gicon = g_file_info_get_icon(regular_icon->priv->file_info);
+        if(G_IS_ICON(gicon))
+            g_object_ref(gicon);
+    }
+
+    g_object_set(file_icon, "gicon", gicon, NULL);
+
+    /* Add any user set emblems */
+    gicon = xfdesktop_file_icon_add_emblems(file_icon);
+
+    /* load the read only emblem if necessary */
+    if(!g_file_info_get_attribute_boolean(regular_icon->priv->file_info,
+                                          G_FILE_ATTRIBUTE_ACCESS_CAN_WRITE))
+    {
+        GIcon *themed_icon = g_themed_icon_new(EMBLEM_READONLY);
+        GEmblem *emblem = g_emblem_new(themed_icon);
+
+        g_emblemed_icon_add_emblem(G_EMBLEMED_ICON(gicon), emblem);
+
+        g_object_unref(emblem);
+        g_object_unref(themed_icon);
+    }
+
+    /* load the symlink emblem if necessary */
+    if(g_file_info_get_attribute_boolean(regular_icon->priv->file_info,
+                                         G_FILE_ATTRIBUTE_STANDARD_IS_SYMLINK))
+    {
+        GIcon *themed_icon = g_themed_icon_new(EMBLEM_SYMLINK);
+        GEmblem *emblem = g_emblem_new(themed_icon);
+
+        g_emblemed_icon_add_emblem(G_EMBLEMED_ICON(gicon), emblem);
+
+        g_object_unref(emblem);
+        g_object_unref(themed_icon);
+    }
+
+    return gicon;
 }
 
 static GdkPixbuf *
 xfdesktop_regular_file_icon_peek_pixbuf(XfdesktopIcon *icon,
-                                        gint size)
+                                        gint width, gint height)
 {
-    XfdesktopRegularFileIcon *file_icon = XFDESKTOP_REGULAR_FILE_ICON(icon);
-    gchar *icon_name = NULL;
-    GdkPixbuf *emblem_pix = NULL;
+    XfdesktopRegularFileIcon *regular_icon = XFDESKTOP_REGULAR_FILE_ICON(icon);
+    GIcon *gicon = NULL;
+    GdkPixbuf *pix = NULL;
 
-    if(size != file_icon->priv->cur_pix_size)
-        xfdesktop_regular_file_icon_invalidate_pixbuf(file_icon);
+    if(!xfdesktop_file_icon_has_gicon(XFDESKTOP_FILE_ICON(icon)))
+        gicon = xfdesktop_regular_file_icon_load_icon(icon);
+    else
+        g_object_get(XFDESKTOP_FILE_ICON(icon), "gicon", &gicon, NULL);
 
-    if(!file_icon->priv->pix) {
-        GIcon *gicon = NULL;
+    pix = xfdesktop_file_utils_get_icon(gicon, width, height,
+                                        regular_icon->priv->pix_opacity);
 
-        /* create a GFile for the $HOME/.thumbnails/ directory */
-        gchar *thumbnail_dir_path = g_build_filename(xfce_get_homedir(), 
-                                                     ".thumbnails", NULL);
-        GFile *thumbnail_dir = g_file_new_for_path(thumbnail_dir_path);
-
-        if(g_file_has_prefix(file_icon->priv->file, thumbnail_dir)) {
-            /* use the filename as custom icon name for thumbnails */
-            icon_name = g_file_get_path(file_icon->priv->file);
-
-            /* release thumbnail path */
-            g_object_unref(thumbnail_dir);
-            g_free(thumbnail_dir_path);
-        } else if(xfdesktop_file_utils_is_desktop_file(file_icon->priv->file_info)) {
-            gchar *contents;
-            gsize length;
-
-            /* try to load the file into memory */
-            if(g_file_load_contents(file_icon->priv->file, NULL, &contents, &length, 
-                                    NULL, NULL)) 
-            {
-                /* allocate a new key file */
-                GKeyFile *key_file = g_key_file_new();
-
-                /* try to parse the key file from the contents of the file */
-                if (g_key_file_load_from_data(key_file, contents, length, 0, NULL)) {
-                    /* try to determine the custom icon name */
-                    icon_name = g_key_file_get_string(key_file,
-                                                      G_KEY_FILE_DESKTOP_GROUP,
-                                                      G_KEY_FILE_DESKTOP_KEY_ICON,
-                                                      NULL);
-                }
-
-                /* free key file and in-memory data */
-                g_key_file_free(key_file);
-                g_free(contents);
-            }
-        } else {
-            /* If we have a thumbnail then they are enabled, use it. */
-            if(file_icon->priv->thumbnail_file)
-            {
-                file_icon->priv->pix = gdk_pixbuf_new_from_file_at_size(g_file_get_path(file_icon->priv->thumbnail_file),
-                                                                        size, size,
-                                                                        NULL);
-            }
-        }
-
-        /* load the symlink emblem if necessary */
-        if(g_file_info_get_attribute_boolean(file_icon->priv->file_info, 
-                                             G_FILE_ATTRIBUTE_STANDARD_IS_SYMLINK)) 
-        {
-            GtkIconTheme *itheme = gtk_icon_theme_get_default();
-            gint sym_pix_size = size * 2 / 3;
-            
-            emblem_pix = gtk_icon_theme_load_icon(itheme, EMBLEM_SYMLINK,
-                                                  sym_pix_size, ITHEME_FLAGS,
-                                                  NULL);
-            if(emblem_pix) {
-                if(gdk_pixbuf_get_width(emblem_pix) != sym_pix_size
-                   || gdk_pixbuf_get_height(emblem_pix) != sym_pix_size)
-                {
-                    GdkPixbuf *tmp = gdk_pixbuf_scale_simple(emblem_pix,
-                                                             sym_pix_size,
-                                                             sym_pix_size,
-                                                             GDK_INTERP_BILINEAR);
-                    g_object_unref(emblem_pix);
-                    emblem_pix = tmp;
-                }
-            }
-        }
-
-        if(file_icon->priv->file_info)
-            gicon = g_file_info_get_icon(file_icon->priv->file_info);
-
-        if(file_icon->priv->pix) {
-            if(emblem_pix) {
-                gint emblem_pix_size = gdk_pixbuf_get_width(emblem_pix);
-                gint dest_size = size - emblem_pix_size;
-
-                /* We have to add the emblem */
-                gdk_pixbuf_composite(emblem_pix, file_icon->priv->pix,
-                                     dest_size, dest_size,
-                                     emblem_pix_size, emblem_pix_size,
-                                     dest_size, dest_size,
-                                     1.0, 1.0, GDK_INTERP_BILINEAR, 255);
-            }
-        } else {
-            file_icon->priv->pix = xfdesktop_file_utils_get_icon(icon_name, gicon,
-                                                                 size, emblem_pix,
-                                                                 file_icon->priv->pix_opacity);
-        }
-        
-        file_icon->priv->cur_pix_size = size;
-
-        if(emblem_pix)
-             g_object_unref(emblem_pix);
-        
-        g_free(icon_name);
-    }
-    
-    return file_icon->priv->pix;
+    return pix;
 }
 
-static G_CONST_RETURN gchar *
+static GdkPixbuf *
+xfdesktop_regular_file_icon_peek_tooltip_pixbuf(XfdesktopIcon *icon,
+                                                gint width, gint height)
+{
+    GIcon *gicon = NULL;
+    GdkPixbuf *tooltip_pix = NULL;
+
+    if(!xfdesktop_file_icon_has_gicon(XFDESKTOP_FILE_ICON(icon)))
+        gicon = xfdesktop_regular_file_icon_load_icon(icon);
+    else
+        g_object_get(XFDESKTOP_FILE_ICON(icon), "gicon", &gicon, NULL);
+
+    tooltip_pix = xfdesktop_file_utils_get_icon(gicon, width, height, 100);
+
+    return tooltip_pix;
+}
+
+static const gchar *
 xfdesktop_regular_file_icon_peek_label(XfdesktopIcon *icon)
 {
     XfdesktopRegularFileIcon *regular_file_icon = XFDESKTOP_REGULAR_FILE_ICON(icon);
@@ -372,6 +535,19 @@ xfdesktop_regular_file_icon_peek_label(XfdesktopIcon *icon)
     return regular_file_icon->priv->display_name;
 }
 
+static gchar *
+xfdesktop_regular_file_icon_get_identifier(XfdesktopIcon *icon)
+{
+    XfdesktopFileIcon *file_icon = XFDESKTOP_FILE_ICON(icon);
+
+    g_return_val_if_fail(XFDESKTOP_IS_FILE_ICON(icon), NULL);
+
+    if(xfdesktop_file_icon_peek_file(file_icon) == NULL)
+        return NULL;
+
+    return g_file_get_path(xfdesktop_file_icon_peek_file(file_icon));
+}
+
 static GdkDragAction
 xfdesktop_regular_file_icon_get_allowed_drag_actions(XfdesktopIcon *icon)
 {
@@ -411,24 +587,36 @@ xfdesktop_regular_file_icon_get_allowed_drag_actions(XfdesktopIcon *icon)
 }
 
 static GdkDragAction
-xfdesktop_regular_file_icon_get_allowed_drop_actions(XfdesktopIcon *icon)
+xfdesktop_regular_file_icon_get_allowed_drop_actions(XfdesktopIcon *icon,
+                                                     GdkDragAction *suggested_action)
 {
     GFileInfo *info = xfdesktop_file_icon_peek_file_info(XFDESKTOP_FILE_ICON(icon));
     
-    if(!info)
+    if(!info) {
+        if(suggested_action)
+            *suggested_action = 0;
         return 0;
+    }
     
     /* if it's executable we can 'copy'.  if it's a folder we can do anything
      * if it's writable. */
     if(g_file_info_get_file_type(info) == G_FILE_TYPE_DIRECTORY) {
-        if(g_file_info_get_attribute_boolean(info, G_FILE_ATTRIBUTE_ACCESS_CAN_WRITE))
+        if(g_file_info_get_attribute_boolean(info, G_FILE_ATTRIBUTE_ACCESS_CAN_WRITE)) {
+            if(suggested_action)
+                *suggested_action = GDK_ACTION_MOVE;
             return GDK_ACTION_MOVE | GDK_ACTION_COPY | GDK_ACTION_LINK;
+        }
     } else {
         if(xfdesktop_file_utils_file_is_executable(info)) {
+            if(suggested_action)
+                *suggested_action = GDK_ACTION_COPY;
             return GDK_ACTION_COPY;
         }
     }
 
+    if(suggested_action)
+        *suggested_action = 0;
+
     return 0;
 }
 
@@ -446,7 +634,7 @@ xfdesktop_regular_file_icon_do_drop_dest(XfdesktopIcon *icon,
     DBG("entering");
     
     g_return_val_if_fail(regular_file_icon && src_file_icon, FALSE);
-    g_return_val_if_fail(xfdesktop_regular_file_icon_get_allowed_drop_actions(icon) != 0,
+    g_return_val_if_fail(xfdesktop_regular_file_icon_get_allowed_drop_actions(icon, NULL) != 0,
                          FALSE);
     
     src_file = xfdesktop_file_icon_peek_file(src_file_icon);
@@ -509,7 +697,7 @@ xfdesktop_regular_file_icon_do_drop_dest(XfdesktopIcon *icon,
     return result;
 }
 
-static G_CONST_RETURN gchar *
+static const gchar *
 xfdesktop_regular_file_icon_peek_tooltip(XfdesktopIcon *icon)
 {
     XfdesktopRegularFileIcon *regular_file_icon = XFDESKTOP_REGULAR_FILE_ICON(icon);
@@ -542,11 +730,8 @@ xfdesktop_regular_file_icon_peek_tooltip(XfdesktopIcon *icon)
 
         size = g_file_info_get_attribute_uint64(info,
                                                 G_FILE_ATTRIBUTE_STANDARD_SIZE);
-#if GLIB_CHECK_VERSION (2, 30, 0)
+
         size_string = g_format_size(size);
-#else
-        size_string = g_format_size_for_display(size);
-#endif
 
         mtime = g_file_info_get_attribute_uint64(info,
                                                  G_FILE_ATTRIBUTE_TIME_MODIFIED);
@@ -689,17 +874,57 @@ xfdesktop_regular_file_icon_update_file_info(XfdesktopFileIcon *icon,
     regular_file_icon->priv->tooltip = NULL;
     
     /* not really easy to check if this changed or not, so just invalidate it */
-    xfdesktop_regular_file_icon_invalidate_pixbuf(regular_file_icon);
+    xfdesktop_file_icon_invalidate_icon(XFDESKTOP_FILE_ICON(icon));
+    xfdesktop_icon_invalidate_pixbuf(XFDESKTOP_ICON(icon));
     xfdesktop_icon_pixbuf_changed(XFDESKTOP_ICON(icon));
 }
 
+static void
+cb_folder_contents_changed(GFileMonitor     *monitor,
+                           GFile            *file,
+                           GFile            *other_file,
+                           GFileMonitorEvent event,
+                           gpointer          user_data)
+{
+    XfdesktopRegularFileIcon *regular_file_icon;
+    gchar *thumbnail_file = NULL;
+
+    if(!user_data || !XFDESKTOP_IS_REGULAR_FILE_ICON(user_data))
+        return;
+
+    regular_file_icon = XFDESKTOP_REGULAR_FILE_ICON(user_data);
+
+    /* not showing thumbnails */
+    if(!regular_file_icon->priv->show_thumbnails)
+        return;
+
+    /* Already has a thumbnail */
+    if(regular_file_icon->priv->thumbnail_file != NULL)
+        return;
+
+    switch(event) {
+        case G_FILE_MONITOR_EVENT_CREATED:
+                thumbnail_file = xfdesktop_load_icon_location_from_folder(XFDESKTOP_FILE_ICON(regular_file_icon));
+                if(thumbnail_file) {
+                    GFile *thumbnail = g_file_new_for_path(thumbnail_file);
+                    /* found a thumbnail file, apply it */
+                    xfdesktop_regular_file_icon_set_thumbnail_file(XFDESKTOP_ICON(regular_file_icon),
+                                                                   thumbnail);
+                    g_free(thumbnail_file);
+                }
+            break;
+        default:
+            break;
+    }
+}
 
 /* public API */
 
 XfdesktopRegularFileIcon *
 xfdesktop_regular_file_icon_new(GFile *file,
                                 GFileInfo *file_info,
-                                GdkScreen *screen)
+                                GdkScreen *screen,
+                                XfdesktopFileIconManager *fmanager)
 {
     XfdesktopRegularFileIcon *regular_file_icon;
 
@@ -729,11 +954,31 @@ xfdesktop_regular_file_icon_new(GFile *file,
 
     regular_file_icon->priv->gscreen = screen;
 
+    regular_file_icon->priv->fmanager = fmanager;
+
     g_signal_connect_swapped(G_OBJECT(gtk_icon_theme_get_for_screen(screen)),
                              "changed",
-                             G_CALLBACK(xfdesktop_regular_file_icon_invalidate_pixbuf),
+                             G_CALLBACK(xfdesktop_icon_invalidate_pixbuf),
                              regular_file_icon);
-    
+
+    if(g_file_info_get_file_type(regular_file_icon->priv->file_info) == G_FILE_TYPE_DIRECTORY) {
+        regular_file_icon->priv->monitor = g_file_monitor(regular_file_icon->priv->file,
+                                                          G_FILE_MONITOR_NONE,
+                                                          NULL,
+                                                          NULL);
+
+        g_signal_connect(regular_file_icon->priv->monitor, "changed",
+                         G_CALLBACK(cb_folder_contents_changed),
+                         regular_file_icon);
+
+        g_object_get(regular_file_icon->priv->fmanager,
+                     "show-thumbnails", &regular_file_icon->priv->show_thumbnails,
+                     NULL);
+
+        /* Keep an eye on the show-thumbnails property for folder thumbnails */
+        g_signal_connect(G_OBJECT(fmanager), "notify::show-thumbnails",
+                         G_CALLBACK(cb_show_thumbnails_notify), regular_file_icon);
+    }
     return regular_file_icon;
 }
 
@@ -748,6 +993,6 @@ xfdesktop_regular_file_icon_set_pixbuf_opacity(XfdesktopRegularFileIcon *icon,
     
     icon->priv->pix_opacity = opacity;
     
-    xfdesktop_regular_file_icon_invalidate_pixbuf(icon);
+    xfdesktop_icon_invalidate_pixbuf(XFDESKTOP_ICON(icon));
     xfdesktop_icon_pixbuf_changed(XFDESKTOP_ICON(icon));
 }
