@@ -16,8 +16,8 @@
  *  GNU Library General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *  along with this program; if not, write to the Free Software Foundation,
+ *  Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
  */
 
 #ifdef HAVE_CONFIG_H
@@ -55,9 +55,7 @@
 struct _XfdesktopSpecialFileIconPrivate
 {
     XfdesktopSpecialFileIconType type;
-    GdkPixbuf *pix;
     gchar *tooltip;
-    gint cur_pix_size;
     GFileMonitor *monitor;
     GFileInfo *file_info;
     GFileInfo *filesystem_info;
@@ -71,11 +69,15 @@ struct _XfdesktopSpecialFileIconPrivate
 static void xfdesktop_special_file_icon_finalize(GObject *obj);
 
 static GdkPixbuf *xfdesktop_special_file_icon_peek_pixbuf(XfdesktopIcon *icon,
-                                                          gint size);
-static G_CONST_RETURN gchar *xfdesktop_special_file_icon_peek_label(XfdesktopIcon *icon);
-static G_CONST_RETURN gchar *xfdesktop_special_file_icon_peek_tooltip(XfdesktopIcon *icon);
+                                                          gint width, gint height);
+static const gchar *xfdesktop_special_file_icon_peek_label(XfdesktopIcon *icon);
+static gchar *xfdesktop_special_file_icon_get_identifier(XfdesktopIcon *icon);
+static GdkPixbuf *xfdesktop_special_file_icon_peek_tooltip_pixbuf(XfdesktopIcon *icon,
+                                                                  gint width, gint height);
+static const gchar *xfdesktop_special_file_icon_peek_tooltip(XfdesktopIcon *icon);
 static GdkDragAction xfdesktop_special_file_icon_get_allowed_drag_actions(XfdesktopIcon *icon);
-static GdkDragAction xfdesktop_special_file_icon_get_allowed_drop_actions(XfdesktopIcon *icon);
+static GdkDragAction xfdesktop_special_file_icon_get_allowed_drop_actions(XfdesktopIcon *icon,
+                                                                          GdkDragAction *suggested_action);
 static gboolean xfdesktop_special_file_icon_do_drop_dest(XfdesktopIcon *icon,
                                                          XfdesktopIcon *src_icon,
                                                          GdkDragAction action);
@@ -94,13 +96,7 @@ static void xfdesktop_special_file_icon_update_trash_count(XfdesktopSpecialFileI
 
 #ifdef HAVE_THUNARX
 static void xfdesktop_special_file_icon_tfi_init(ThunarxFileInfoIface *iface);
-#endif
 
-
-static inline void xfdesktop_special_file_icon_invalidate_pixbuf(XfdesktopSpecialFileIcon *icon);
-
-
-#ifdef HAVE_THUNARX
 G_DEFINE_TYPE_EXTENDED(XfdesktopSpecialFileIcon, xfdesktop_special_file_icon,
                        XFDESKTOP_TYPE_FILE_ICON, 0,
                        G_IMPLEMENT_INTERFACE(THUNARX_TYPE_FILE_INFO,
@@ -126,6 +122,8 @@ xfdesktop_special_file_icon_class_init(XfdesktopSpecialFileIconClass *klass)
     
     icon_class->peek_pixbuf = xfdesktop_special_file_icon_peek_pixbuf;
     icon_class->peek_label = xfdesktop_special_file_icon_peek_label;
+    icon_class->get_identifier = xfdesktop_special_file_icon_get_identifier;
+    icon_class->peek_tooltip_pixbuf = xfdesktop_special_file_icon_peek_tooltip_pixbuf;
     icon_class->peek_tooltip = xfdesktop_special_file_icon_peek_tooltip;
     icon_class->get_allowed_drag_actions = xfdesktop_special_file_icon_get_allowed_drag_actions;
     icon_class->get_allowed_drop_actions = xfdesktop_special_file_icon_get_allowed_drop_actions;
@@ -154,12 +152,9 @@ xfdesktop_special_file_icon_finalize(GObject *obj)
     GtkIconTheme *itheme = gtk_icon_theme_get_for_screen(icon->priv->gscreen);
     
     g_signal_handlers_disconnect_by_func(G_OBJECT(itheme),
-                                         G_CALLBACK(xfdesktop_special_file_icon_invalidate_pixbuf),
+                                         G_CALLBACK(xfdesktop_icon_invalidate_pixbuf),
                                          icon);
 
-    if(icon->priv->pix)
-        g_object_unref(G_OBJECT(icon->priv->pix));
-
     if(icon->priv->monitor) {
         g_signal_handlers_disconnect_by_func(icon->priv->monitor,
                                              G_CALLBACK(xfdesktop_special_file_icon_changed),
@@ -171,7 +166,7 @@ xfdesktop_special_file_icon_finalize(GObject *obj)
 
     if(icon->priv->file_info)
         g_object_unref(icon->priv->file_info);
-    
+
     if(icon->priv->tooltip)
         g_free(icon->priv->tooltip);
     
@@ -206,62 +201,91 @@ xfdesktop_special_file_icon_tfi_init(ThunarxFileInfoIface *iface)
 }
 #endif  /* HAVE_THUNARX */
 
-static inline void
-xfdesktop_special_file_icon_invalidate_pixbuf(XfdesktopSpecialFileIcon *icon)
-{
-    if(icon->priv->pix) {
-        g_object_unref(G_OBJECT(icon->priv->pix));
-        icon->priv->pix = NULL;
-    }
-}
 
-static GdkPixbuf *
-xfdesktop_special_file_icon_peek_pixbuf(XfdesktopIcon *icon,
-                                        gint size)
+static GIcon *
+xfdesktop_special_file_icon_load_icon(XfdesktopIcon *icon)
 {
-    XfdesktopSpecialFileIcon *file_icon = XFDESKTOP_SPECIAL_FILE_ICON(icon);
-    GIcon *gicon = NULL;
-    const gchar *custom_icon_name = NULL;
+    XfdesktopSpecialFileIcon *special_icon = XFDESKTOP_SPECIAL_FILE_ICON(icon);
+    XfdesktopFileIcon *file_icon = XFDESKTOP_FILE_ICON(icon);
+    gchar *icon_name = NULL;
     GFile *parent = NULL;
+    GIcon *gicon = NULL;
 
-    if(size != file_icon->priv->cur_pix_size)
-        xfdesktop_special_file_icon_invalidate_pixbuf(file_icon);
-
-    /* Already have a good icon */
-    if(file_icon->priv->pix != NULL)
-        return file_icon->priv->pix;
+    TRACE("entering");
 
     /* use a custom icon name for the local filesystem root */
-    parent = g_file_get_parent(file_icon->priv->file);
-    if(!parent && g_file_has_uri_scheme(file_icon->priv->file, "file"))
-        custom_icon_name = "drive-harddisk";
+    parent = g_file_get_parent(special_icon->priv->file);
+    if(!parent && g_file_has_uri_scheme(special_icon->priv->file, "file"))
+        icon_name = g_strdup("drive-harddisk");
+
     if(parent)
         g_object_unref(parent);
 
     /* use a custom icon for the trash, based on it having files
      * the user can delete */
-    if(file_icon->priv->type == XFDESKTOP_SPECIAL_FILE_ICON_TRASH) {
-        if(file_icon->priv->trash_item_count == 0)
-            custom_icon_name = "user-trash";
+    if(special_icon->priv->type == XFDESKTOP_SPECIAL_FILE_ICON_TRASH) {
+        if(special_icon->priv->trash_item_count == 0)
+            icon_name = g_strdup("user-trash");
         else
-            custom_icon_name = "user-trash-full";
+            icon_name = g_strdup("user-trash-full");
     }
 
-    if(file_icon->priv->file_info)
-        gicon = g_file_info_get_icon(file_icon->priv->file_info);
+    /* Create the themed icon for it */
+    if(icon_name) {
+        gicon = g_themed_icon_new(icon_name);
+        g_free(icon_name);
+    }
 
-    file_icon->priv->pix = xfdesktop_file_utils_get_icon(custom_icon_name,
-                                                         gicon,
-                                                         size,
-                                                         NULL,
-                                                         100);
+    /* If we still don't have an icon, use the default */
+    if(!G_IS_ICON(gicon)) {
+        gicon = g_file_info_get_icon(special_icon->priv->file_info);
+        if(G_IS_ICON(gicon))
+            g_object_ref(gicon);
+    }
 
-    file_icon->priv->cur_pix_size = size;
-    
-    return file_icon->priv->pix;
+    g_object_set(file_icon, "gicon", gicon, NULL);
+
+    /* Add any user set emblems */
+    gicon = xfdesktop_file_icon_add_emblems(file_icon);
+
+    return gicon;
 }
 
-static G_CONST_RETURN gchar *
+static GdkPixbuf *
+xfdesktop_special_file_icon_peek_pixbuf(XfdesktopIcon *icon,
+                                        gint width, gint height)
+{
+    GIcon *gicon = NULL;
+    GdkPixbuf *pix = NULL;
+
+    if(!xfdesktop_file_icon_has_gicon(XFDESKTOP_FILE_ICON(icon)))
+        gicon = xfdesktop_special_file_icon_load_icon(icon);
+    else
+        g_object_get(XFDESKTOP_FILE_ICON(icon), "gicon", &gicon, NULL);
+
+    pix = xfdesktop_file_utils_get_icon(gicon, height, height, 100);
+
+    return pix;
+}
+
+static GdkPixbuf *
+xfdesktop_special_file_icon_peek_tooltip_pixbuf(XfdesktopIcon *icon,
+                                                gint width, gint height)
+{
+    GIcon *gicon = NULL;
+    GdkPixbuf *tooltip_pix = NULL;
+
+    if(!xfdesktop_file_icon_has_gicon(XFDESKTOP_FILE_ICON(icon)))
+        gicon = xfdesktop_special_file_icon_load_icon(icon);
+    else
+        g_object_get(XFDESKTOP_FILE_ICON(icon), "gicon", &gicon, NULL);
+
+    tooltip_pix = xfdesktop_file_utils_get_icon(gicon, height, height, 100);
+
+    return tooltip_pix;
+}
+
+static const gchar *
 xfdesktop_special_file_icon_peek_label(XfdesktopIcon *icon)
 {
     XfdesktopSpecialFileIcon *special_file_icon = XFDESKTOP_SPECIAL_FILE_ICON(icon);
@@ -275,6 +299,19 @@ xfdesktop_special_file_icon_peek_label(XfdesktopIcon *icon)
         return info ? g_file_info_get_display_name(info) : NULL;
 }
 
+static gchar *
+xfdesktop_special_file_icon_get_identifier(XfdesktopIcon *icon)
+{
+    XfdesktopFileIcon *file_icon = XFDESKTOP_FILE_ICON(icon);
+
+    g_return_val_if_fail(XFDESKTOP_IS_FILE_ICON(icon), NULL);
+
+    if(xfdesktop_file_icon_peek_file(file_icon) == NULL)
+        return NULL;
+
+    return g_file_get_path(xfdesktop_file_icon_peek_file(file_icon));
+}
+
 static GdkDragAction
 xfdesktop_special_file_icon_get_allowed_drag_actions(XfdesktopIcon *icon)
 {
@@ -305,7 +342,8 @@ xfdesktop_special_file_icon_get_allowed_drag_actions(XfdesktopIcon *icon)
 }
 
 static GdkDragAction
-xfdesktop_special_file_icon_get_allowed_drop_actions(XfdesktopIcon *icon)
+xfdesktop_special_file_icon_get_allowed_drop_actions(XfdesktopIcon *icon,
+                                                     GdkDragAction *suggested_action)
 {
     XfdesktopSpecialFileIcon *special_file_icon = XFDESKTOP_SPECIAL_FILE_ICON(icon);
     GFileInfo *info;
@@ -319,13 +357,20 @@ xfdesktop_special_file_icon_get_allowed_drop_actions(XfdesktopIcon *icon)
             {
                 DBG("can move, copy and link");
                 actions = GDK_ACTION_MOVE | GDK_ACTION_COPY | GDK_ACTION_LINK;
+                if(suggested_action)
+                    *suggested_action = GDK_ACTION_MOVE;
             }
         }
     } else {
         DBG("can move");
         actions = GDK_ACTION_MOVE; /* everything else is just silly */
+        if(suggested_action)
+            *suggested_action = GDK_ACTION_MOVE;
     }
     
+    if(suggested_action)
+        *suggested_action = 0;
+
     return actions;
 }
 
@@ -344,7 +389,7 @@ xfdesktop_special_file_icon_do_drop_dest(XfdesktopIcon *icon,
     DBG("entering");
     
     g_return_val_if_fail(special_file_icon && src_file_icon, FALSE);
-    g_return_val_if_fail(xfdesktop_special_file_icon_get_allowed_drop_actions(icon),
+    g_return_val_if_fail(xfdesktop_special_file_icon_get_allowed_drop_actions(icon, NULL),
                          FALSE);
     
     src_file = xfdesktop_file_icon_peek_file(src_file_icon);
@@ -401,7 +446,7 @@ xfdesktop_special_file_icon_do_drop_dest(XfdesktopIcon *icon,
     return result;
 }
 
-static G_CONST_RETURN gchar *
+static const gchar *
 xfdesktop_special_file_icon_peek_tooltip(XfdesktopIcon *icon)
 {
     XfdesktopSpecialFileIcon *special_file_icon = XFDESKTOP_SPECIAL_FILE_ICON(icon);
@@ -439,11 +484,8 @@ xfdesktop_special_file_icon_peek_tooltip(XfdesktopIcon *icon)
 
             size = g_file_info_get_attribute_uint64(info,
                                                     G_FILE_ATTRIBUTE_STANDARD_SIZE);
-#if GLIB_CHECK_VERSION (2, 30, 0)
+
             size_string = g_format_size(size);
-#else
-            size_string = g_format_size_for_display(size);
-#endif
 
             mtime = g_file_info_get_attribute_uint64(info,
                                                      G_FILE_ATTRIBUTE_TIME_MODIFIED);
@@ -566,6 +608,12 @@ xfdesktop_special_file_icon_changed(GFileMonitor *monitor,
     g_return_if_fail(G_IS_FILE(file));
     g_return_if_fail(XFDESKTOP_IS_SPECIAL_FILE_ICON(special_file_icon));
 
+    /* We don't care about change events only created/deleted */
+    if(event == G_FILE_MONITOR_EVENT_CHANGED ||
+       event == G_FILE_MONITOR_EVENT_ATTRIBUTE_CHANGED ||
+       event == G_FILE_MONITOR_EVENT_CHANGES_DONE_HINT)
+        return;
+
     /* release the old file information */
     if(special_file_icon->priv->file_info) {
         g_object_unref(special_file_icon->priv->file_info);
@@ -598,7 +646,8 @@ xfdesktop_special_file_icon_changed(GFileMonitor *monitor,
     special_file_icon->priv->tooltip = NULL;
 
     /* update the icon */
-    xfdesktop_special_file_icon_invalidate_pixbuf(special_file_icon);
+    xfdesktop_file_icon_invalidate_icon(XFDESKTOP_FILE_ICON(special_file_icon));
+    xfdesktop_icon_invalidate_pixbuf(XFDESKTOP_ICON(special_file_icon));
     xfdesktop_icon_pixbuf_changed(XFDESKTOP_ICON(special_file_icon));
 }
 
@@ -617,13 +666,6 @@ xfdesktop_special_file_icon_update_trash_count(XfdesktopSpecialFileIcon *special
         return;
     }
 
-    special_file_icon->priv->trash_item_count = g_file_info_get_attribute_uint32(
-                                                    special_file_icon->priv->file_info,
-                                                    G_FILE_ATTRIBUTE_TRASH_ITEM_COUNT);
-
-    if(special_file_icon->priv->trash_item_count == 0)
-        return;
-
     /* The trash count may return a number of files the user can't
      * currently delete, for example if the file is in a removable
      * drive that isn't mounted.
@@ -648,6 +690,7 @@ xfdesktop_special_file_icon_update_trash_count(XfdesktopSpecialFileIcon *special
     g_object_unref(enumerator);
 
     special_file_icon->priv->trash_item_count = n;
+    TRACE("exiting, trash count %d", n);
 }
 
 /* public API */
@@ -701,7 +744,7 @@ xfdesktop_special_file_icon_new(XfdesktopSpecialFileIconType type,
 
     g_signal_connect_swapped(G_OBJECT(gtk_icon_theme_get_for_screen(screen)),
                              "changed",
-                             G_CALLBACK(xfdesktop_special_file_icon_invalidate_pixbuf),
+                             G_CALLBACK(xfdesktop_icon_invalidate_pixbuf),
                              special_file_icon);
 
     special_file_icon->priv->monitor = g_file_monitor(special_file_icon->priv->file,
